<?php
namespace DistributeHelper;
error_reporting(E_ALL & ~E_NOTICE);
include "Utility.php";

class PackageClass
{
	//当所有包都安装完毕后
	public static function index($vendorDir,$repoName){
		//创建.distribute目录
		$distributeDir = getDistributeDir($vendorDir);
		if( !file_exists($distributeDir) ){
			mkdir($distributeDir,0777,true);
		}
		
		//设置日志文件路径
		if(! defined(LOG_PATH)){
			define('LOG_PATH',getLogPath($distributeDir))
		}
				
		//获取配置信息
		$cfgPath = getConfigPath($distributeDir);
		$config = loadConfig($cfgPath);
		if(!$config){
			printf("Failed to load distribute config -- %s\n",$cfgPath);
			exit();
		}
		
		//获取当前包信息
		$packageInfo = self::getPackageInfo($config,$vendorDir,$repoName);
		if(!$packageInfo){
			exit();
		}
		
		//获取当前包的module文件
        self::fetchModuleFiles($packageInfo);
		
		//获取当前包的common文件
		self::fetchCommonFiles($packageInfo);

		//获取当前包的config文件
		self::fetchConfigFiles($packageInfo);
		
		//顶层包的额外工作
		if($repoName == ""){
			//注册全局路径
			self::registerPath($packageInfo);
			//注册自启动程序
			self::registerAutorun($packageInfo);
		}
	}

	//获取包相关信息
	private static function getPackageInfo($config,$vendorDir, $repoName){
		$rootDir = dirname($vendorDir);
		if($repoName == ""){ 
			$repoDir = $rootDir; //最上层包所在目录
		}
		else{
			$repoDir = pathJoin($vendorDir,$repoName);//其他依赖包所在目录
		}
		
		//解析composer.json
		$cfgPath = pathJoin($repoDir,'composer.json');
		$data = json_decode(file_get_contents($cfgPath), true);
		if(!$data["version"]){
			logError('no version node exists',$cfgPath);
			return false;
		}
		$moduleName = $data["distribute-helper"]["module-name"];
		if(!$moduleName){
			logError('no distribute-helper/module-name node exists',$cfgPath);
			return false;
		}
		
		//获取安装目录
		$commonDir = pathJoin($rootDir,'common');
		$configDir = pathJoin($rootDir,'config',$moduleName);
		$moduleDir = pathJoin($rootDir,'module',$moduleName . '-' . $data["version"]);
		
		//获取缓存目录
		$cacheName = str_replace('/','-',$data["name"]) . '-' . $data["version"];
		$cacheDir = pathJoin($config['cache-dir'],$cacheName);
		
		//返回包的相关信息
		return array(
			"repoName" => $data["name"],
			"repoDir"  => $repoDir,
			"cacheDir" => $cacheDir,
			"ossHost" => $config['oss-host'],
			"moduleName" => $moduleName,
			"commonDir" => $commonDir,
			"configDir" => $configDir,
			"moduleDir" => $moduleDir,
			"helper" => $data["distribute-helper"],
		);
	}
	
	//下载包module文件
	private static function fetchModuleFiles($packageInfo){
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$moduleDir = $packageInfo['moduleDir'];
		$ossHost = $packageInfo['ossHost'];
		
		$moduleFiles = $packageInfo['helper']['module-files'];
		if(! $moduleFiles){
			logInfo($repoName,'no module files defined. skip.');
			return;
		}
		
		foreach($moduleFiles as $fileName){
			printf("[%s] fetching file -- %s\n",$repoName,$fileName);

			//下载文件到缓存区
			$filePath = downloadToDir($ossHost,$repoName,$fileName,$cacheDir);
			if(! $filePath ){
				printf("[%s] fetch file failed -- %s\n",$repoName,$fileName);
				continue;
			}

			// 解压文件
			if( isZipFile($fileName) ){
				if(! unzipFile($filePath,$moduleDir)) {
					printf("[%s] unzip file failed -- %s\n ",$repoName,$fileName);
					unlink($filePath);
				}
			}
		}
	}
	
	//下载包common文件
	private static function fetchCommonFiles($packageInfo){
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$commonDir = $packageInfo['commonDir'];
		$ossHost = $packageInfo['ossHost'];
		$commonFiles = $packageInfo['helper']['common-files'];
		if(! $commonFiles){
			logInfo($repoName,'no common files defined. skip.');
			return;
		}
		foreach($commonFiles as $fileName){
			printf("[%s] fetching common file -- %s\n",$repoName,$fileName);
			//下载文件到缓存区
			$filePath = downloadToDir($ossHost,$repoName,$fileName,$cacheDir);
			if(! $filePath ){
				printf("[%s] fetch common file failed -- %s\n",$repoName,$fileName);
				continue;
			}

			// 解压文件
			if( isZipFile($fileName) ){
				if(! unzipFile($filePath,$commonDir)) {
					printf("[%s] unzip common file failed -- %s\n ",$repoName,$fileName);
					unlink($filePath);
				}
			}
		}
	}
	
	//下载包config文件
	private static function fetchConfigFiles($packageInfo){
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$configDir = $packageInfo['configDir'];
		$ossHost = $packageInfo['ossHost'];
		$configFiles = $packageInfo['helper']['config-files'];
		if(! $configFiles){
			logInfo($repoName,'no config files defined. skip.');
			return;
		}
		foreach($configFiles as $fileName){
			printf("[%s] fetching config file -- %s\n",$repoName,$fileName);
			//下载文件到缓存区
			$filePath = downloadToDir($ossHost,$repoName,$fileName,$cacheDir);
			if(! $filePath ){
				printf("[%s] fetch config file failed -- %s\n",$repoName,$fileName);
				continue;
			}

			// 解压文件
			if( isZipFile($fileName) ){
				if(! unzipFile($filePath,$configDir)) {
					printf("[%s] unzip config file failed -- %s\n ",$repoName,$fileName);
					unlink($filePath);
				}
			}
		}
	}
	
	private static function registerPath($packageInfo){
	}
	
	private static function registerAutorun($packageInfo){
	}	
}